@@ -17,7 +17,7 @@ short.summary <- function(object) {
                     packageDescription("blavaan", fields="Version")))
     } else if(object@Fit@iterations > 0) {
         if(object@Fit@converged) {
-	    cat(sprintf("blavaan (%s) results of %3i samples after %3i adapt+burnin iterations\n",
+	    cat(sprintf("blavaan (%s) results of %3i samples after %3i adapt/burnin iterations\n",
                     packageDescription("blavaan", fields="Version"),
                     object@external$sample,
                     object@external$burnin))
@@ -192,6 +192,17 @@ function(object, header       = TRUE,
 
 
     if(estimates) {
+        jagtarget <- class(object@external$mcmcout) == "runjags"
+        newpt <- object@ParTable
+        newpt$group[newpt$group == 0] <- 1 # for defined parameters
+
+        if(!jagtarget){
+            rhorows <- which(newpt$mat == "rho" | newpt$mat == "lvrho")
+            if(length(rhorows) > 0){
+                newpt <- lapply(newpt, function(x) x[-rhorows])
+                object@ParTable <- lapply(object@ParTable, function(x) x[-rhorows])
+            }
+        }
         PE <- parameterEstimates(object, se = TRUE, zstat = FALSE,
                                  ci = TRUE,
                                  standardized = standardized,
@@ -203,6 +214,7 @@ function(object, header       = TRUE,
         if(standardized && std.nox) {
             PE$std.all <- PE$std.nox
         }
+        PE$group[PE$group == 0] <- 1
 
         attributes(PE)$information <- "MCMC"
         attributes(PE)$se <- "MCMC"
@@ -211,17 +223,6 @@ function(object, header       = TRUE,
 
         ## TODO put parameter priors in partable
 
-        newpt <- object@ParTable
-        newpt$group[newpt$group == 0] <- 1 # for defined parameters
-        PE$group[PE$group == 0] <- 1
-        jagtarget <- class(object@external$mcmcout) == "runjags"
-        if(!jagtarget){
-            rhorows <- which(newpt$mat == "rho")
-            if(length(rhorows) > 0){
-                newpt <- lapply(newpt, function(x) x[-rhorows])
-            }
-        }
-
         ## match jags names to partable, then partable to PE
         if(jagtarget){
             pte2 <- which(!is.na(newpt$jagpnum))
