@@ -24,12 +24,12 @@ jagpriors <- function(nu="dnorm(0,1e-3)", alpha="dnorm(0,1e-2)",
 }
 
 ## see ?stan::expose_stan_functions for obtaining margloglik info
-stanpriors <- function(nu="normal(0,sqrt(1000))",
+stanpriors <- function(nu="normal(0,1000^.5)",
                        alpha="normal(0,10)", lambda="normal(0,10)",
                        beta="normal(0,10)", itheta="gamma(1,.5)",
                        ipsi="gamma(1,.5)", rho="beta(1,1)",
                        ibpsi="wishart(3,iden)",
-                       tau="normal(0,sqrt(10))",
+                       tau="normal(0,10^.5)",
                        delta="gamma(1,.5)"){
 
   dp <- c(nu=nu, alpha=alpha, lambda=lambda, beta=beta,
